package org.sonatype.sisu.velocity.internal;

import java.util.Collections;
import java.util.List;

import javax.inject.Inject;
import javax.inject.Named;
import javax.inject.Singleton;

import org.apache.velocity.app.VelocityEngine;
import org.apache.velocity.runtime.RuntimeConstants;
import org.apache.velocity.runtime.log.LogChute;
import org.slf4j.Logger;
import org.sonatype.sisu.velocity.Velocity;

@Named
@Singleton
public class VelocityImpl
    implements Velocity
{
    private static final String DEFAULT = "default";

    private final Logger logger;

    private final VelocityEngine velocityEngine;

    public VelocityImpl()
    {
        this( Collections.<VelocityConfigurator> emptyList() );
    }

    @Inject
    public VelocityImpl( final List<VelocityConfigurator> configurators )
    {
        // to log using slf4j, the Sonatype default
        final Slf4jLogChute logChute = new Slf4jLogChute();
        this.logger = logChute.getLogger();
        this.logger.info( "Creating default instance of VelocityEngine" );
        this.velocityEngine = createEngine( DEFAULT, logChute, configurators );
    }

    public VelocityEngine getEngine()
    {
        return velocityEngine;
    }

    public VelocityEngine createEngine( final String id, final LogChute logChute,
                                        final List<VelocityConfigurator> configurators )
    {
        if ( id == null || id.trim().length() == 0 )
        {
            throw new NullPointerException( "Velocity instance ID is empty!" );
        }

        VelocityEngine velocityEngine = new VelocityEngine();
        velocityEngine.setApplicationAttribute( INSTANCE_ID, id );
        if ( logChute != null )
        {
            velocityEngine.setProperty( RuntimeConstants.RUNTIME_LOG_LOGSYSTEM, logChute );
        }

        // setting various defaults
        // ========================
        // to avoid "unable to find resource 'VM_global_library.vm' in any resource loader."
        velocityEngine.setProperty( "velocimacro.library", "" );
        // to use classpath loader
        velocityEngine.setProperty( RuntimeConstants.RESOURCE_LOADER, "class" );
        velocityEngine.setProperty( "class.resource.loader.class",
            "org.apache.velocity.runtime.resource.loader.ClasspathResourceLoader" );
        // to make us strict with template references (early problem detection)
        velocityEngine.setProperty( "runtime.references.strict", "true" );

        // customize if needed
        // ===================
        if ( configurators != null )
        {
            for ( VelocityConfigurator configurator : configurators )
            {
                configurator.configure( velocityEngine );
            }
        }

        // fire up the engine
        // ==================
        try
        {
            velocityEngine.init();
        }
        catch ( Exception e )
        {
            throw new IllegalStateException( "Cannot initialize VelocityEngine", e );
        }

        return velocityEngine;
    }
}
